"""
==========================================
Leaf-scale processes (:mod:`ecoflux.leaf`)
==========================================

.. currentmodule:: ecoflux.leaf

Leaf-scale exchange of mass and energy.

"""
from . import light_response  # noqa
